@@ -1,11 +1,15 @@
 require 'rgen/ecore/ecore'
+require 'puppet/parser/scope'
 require 'puppet/pops/evaluator/compare_operator'
 require 'puppet/pops/evaluator/relationship_operator'
 require 'puppet/pops/evaluator/access_operator'
 require 'puppet/pops/evaluator/closure'
 require 'puppet/pops/evaluator/external_syntax_support'
+require 'puppet/pops/types/iterable'
 
-# This implementation of {Puppet::Pops::Evaluator} performs evaluation using the puppet 3.x runtime system
+module Puppet::Pops
+module Evaluator
+# This implementation of {Evaluator} performs evaluation using the puppet 3.x runtime system
 # in a manner largely compatible with Puppet 3.x, but adds new features and introduces constraints.
 #
 # The evaluation uses _polymorphic dispatch_ which works by dispatching to the first found method named after
@@ -18,44 +22,40 @@ require 'puppet/pops/evaluator/external_syntax_support'
 # either the actual class, or one of its super classes). The _scope_ parameter is always the scope in which
 # the evaluation takes place. If nothing else is mentioned, the return is always the result of evaluation.
 #
-# See {Puppet::Pops::Visitable} and {Puppet::Pops::Visitor} for more information about
+# See {Visitable} and {Visitor} for more information about
 # polymorphic calling.
 #
-class Puppet::Pops::Evaluator::EvaluatorImpl
-  include Puppet::Pops::Utils
+class EvaluatorImpl
+  include Utils
 
   # Provides access to the Puppet 3.x runtime (scope, etc.)
   # This separation has been made to make it easier to later migrate the evaluator to an improved runtime.
   #
-  include Puppet::Pops::Evaluator::Runtime3Support
-  include Puppet::Pops::Evaluator::ExternalSyntaxSupport
+  include Runtime3Support
+  include ExternalSyntaxSupport
 
-  # This constant is not defined as Float::INFINITY in Ruby 1.8.7 (but is available in later version
-  # Refactor when support is dropped for Ruby 1.8.7.
-  #
-  INFINITY = 1.0 / 0.0
   EMPTY_STRING = ''.freeze
   COMMA_SEPARATOR = ', '.freeze
 
   # Reference to Issues name space makes it easier to refer to issues
   # (Issues are shared with the validator).
   #
-  Issues = Puppet::Pops::Issues
+  Issues = Issues
 
   def initialize
-    @@eval_visitor     ||= Puppet::Pops::Visitor.new(self, "eval", 1, 1)
-    @@lvalue_visitor   ||= Puppet::Pops::Visitor.new(self, "lvalue", 1, 1)
-    @@assign_visitor   ||= Puppet::Pops::Visitor.new(self, "assign", 3, 3)
-    @@string_visitor   ||= Puppet::Pops::Visitor.new(self, "string", 1, 1)
+    @@eval_visitor     ||= Visitor.new(self, "eval", 1, 1)
+    @@lvalue_visitor   ||= Visitor.new(self, "lvalue", 1, 1)
+    @@assign_visitor   ||= Visitor.new(self, "assign", 3, 3)
+    @@string_visitor   ||= Visitor.new(self, "string", 1, 1)
 
-    @@type_calculator  ||= Puppet::Pops::Types::TypeCalculator.new()
-    @@type_parser      ||= Puppet::Pops::Types::TypeParser.new()
+    @@type_calculator  ||= Types::TypeCalculator.new()
+    @@type_parser      ||= Types::TypeParser.new()
 
-    @@compare_operator     ||= Puppet::Pops::Evaluator::CompareOperator.new()
-    @@relationship_operator ||= Puppet::Pops::Evaluator::RelationshipOperator.new()
+    @@compare_operator     ||= CompareOperator.new()
+    @@relationship_operator ||= RelationshipOperator.new()
 
     # Use null migration checker unless given in context
-    @migration_checker = (Puppet.lookup(:migration_checker) { Puppet::Pops::Migration::MigrationChecker.new() })
+    @migration_checker = (Puppet.lookup(:migration_checker) { Migration::MigrationChecker.new() })
   end
 
   # @api private
@@ -76,8 +76,8 @@ class Puppet::Pops::Evaluator::EvaluatorImpl
     begin
       @@eval_visitor.visit_this_1(self, target, scope)
 
-    rescue Puppet::Pops::SemanticError => e
-      # A raised issue may not know the semantic target, use errors call stack, but fill in the 
+    rescue SemanticError => e
+      # A raised issue may not know the semantic target, use errors call stack, but fill in the
       # rest from a supplied semantic object, or the target instruction if there is not semantic
       # object.
       #
@@ -123,7 +123,7 @@ class Puppet::Pops::Evaluator::EvaluatorImpl
   #
   # @param target [Object] assignment target - see methods on the pattern assign_TYPE for actual supported types.
   # @param value [Object] the value to assign to `target`
-  # @param o [Puppet::Pops::Model::PopsObject] originating instruction
+  # @param o [Model::PopsObject] originating instruction
   # @param scope [Object] the runtime specific scope where evaluation should take place
   #
   # @api private
@@ -157,12 +157,12 @@ class Puppet::Pops::Evaluator::EvaluatorImpl
   #
   # @param scope [Puppet::Parser::Scope] the parent scope
   # @param variable_bindings [Hash{String => Object}] the variable names and values to bind (names are keys, bound values are values)
-  # @param block [Puppet::Pops::Model::BlockExpression] the sequence of expressions to evaluate in the new scope
+  # @param block [Model::BlockExpression] the sequence of expressions to evaluate in the new scope
   #
   # @api private
   #
   def evaluate_block_with_bindings(scope, variable_bindings, block_expr)
-    with_guarded_scope(scope) do
+    scope.with_guarded_scope do
       # change to create local scope_from - cannot give it file and line -
       # that is the place of the call, not "here"
       create_local_scope_from(variable_bindings, scope)
@@ -183,13 +183,18 @@ class Puppet::Pops::Evaluator::EvaluatorImpl
     fail(Issues::ILLEGAL_ASSIGNMENT, o)
   end
 
+  # An array is assignable if all entries are lvalues
+  def lvalue_LiteralList(o, scope)
+    o.values.map {|x| lvalue(x, scope) }
+  end
+
   # Assign value to named variable.
   # The '$' sign is never part of the name.
   # @example In Puppet DSL
   #   $name = value
   # @param name [String] name of variable without $
   # @param value [Object] value to assign to the variable
-  # @param o [Puppet::Pops::Model::PopsObject] originating instruction
+  # @param o [Model::PopsObject] originating instruction
   # @param scope [Object] the runtime specific scope where evaluation should take place
   # @return [value<Object>]
   #
@@ -211,6 +216,44 @@ class Puppet::Pops::Evaluator::EvaluatorImpl
     fail(Issues::ILLEGAL_ASSIGNMENT, o)
   end
 
+  def assign_Array(lvalues, values, o, scope)
+    if values.is_a?(Hash)
+      lvalues.map do |lval|
+        assign(lval,
+          values.fetch(lval) {|k| fail(Issues::MISSING_MULTI_ASSIGNMENT_KEY, o, :key =>k)},
+          o, scope)
+      end
+    elsif values.is_a?(Puppet::Pops::Types::PHostClassType)
+      # assign variables from class variables
+      # lookup class resource and return one or more parameter values
+      # TODO: behavior when class_name is nil
+      resource = find_resource(scope, 'class', values.class_name)
+      if resource
+        base_name = "#{values.class_name.downcase}::"
+        idx = -1
+        result = lvalues.map do |lval|
+          idx += 1
+          varname = "#{base_name}#{lval}"
+          if variable_exists?(varname, scope)
+            result = get_variable_value(varname, o, scope)
+            assign(lval, result, o, scope)
+          else
+            fail(Puppet::Pops::Issues::MISSING_MULTI_ASSIGNMENT_VARIABLE, o.left_expr.values[idx], {:name => varname})
+          end
+        end
+      else
+        fail(Issues::UNKNOWN_RESOURCE, o.right_expr, {:type_name => 'Class', :title => values.class_name})
+      end
+
+    else
+      values = [values] unless values.is_a?(Array)
+      if values.size != lvalues.size
+        fail(Issues::ILLEGAL_MULTI_ASSIGNMENT_SIZE, o, :expected =>lvalues.size, :actual => values.size)
+      end
+      lvalues.zip(values).map { |lval, val| assign(lval, val, o, scope) }
+    end
+  end
+
   def eval_Factory(o, scope)
     evaluate(o.current, scope)
   end
@@ -240,7 +283,7 @@ class Puppet::Pops::Evaluator::EvaluatorImpl
   #
   def eval_ReservedWord(o, scope)
     if !o.future
-      fail(Puppet::Pops::Issues::RESERVED_WORD, o, {:word => o.word})
+      fail(Issues::RESERVED_WORD, o, {:word => o.word})
     else
       o.word
     end
@@ -257,7 +300,9 @@ class Puppet::Pops::Evaluator::EvaluatorImpl
   # A QualifiedReference (i.e. a  capitalized qualified name such as Foo, or Foo::Bar) evaluates to a PType
   #
   def eval_QualifiedReference(o, scope)
-    @@type_parser.interpret(o)
+    type = @@type_parser.interpret(o, scope)
+    fail(Issues::UNKNOWN_RESOURCE_TYPE, o, {:type_name => type.type_string }) if type.is_a?(Types::PTypeReferenceType)
+    type
   end
 
   def eval_NotExpression(o, scope)
@@ -277,6 +322,8 @@ class Puppet::Pops::Evaluator::EvaluatorImpl
       candidate
     when Hash
       candidate.to_a
+    when Puppet::Pops::Types::Iterator
+      candidate.to_a
     else
       # turns anything else into an array (so result can be unfolded)
       [candidate]
@@ -363,7 +410,7 @@ class Puppet::Pops::Evaluator::EvaluatorImpl
       rescue ZeroDivisionError => e
         fail(Issues::DIV_BY_ZERO, right_o)
       end
-      if result == INFINITY || result == -INFINITY
+      if result == Float::INFINITY || result == -Float::INFINITY
         fail(Issues::RESULT_IS_INFINITY, left_o, {:operator => operator})
       end
       result
@@ -401,8 +448,16 @@ class Puppet::Pops::Evaluator::EvaluatorImpl
   #
   def eval_AccessExpression(o, scope)
     left = evaluate(o.left_expr, scope)
-    keys = o.keys.nil? ? [] : o.keys.collect {|key| evaluate(key, scope) }
-    Puppet::Pops::Evaluator::AccessOperator.new(o).access(left, scope, *keys)
+    keys = o.keys || []
+    if left.is_a?(Types::PHostClassType)
+      # Evaluate qualified references without errors no undefined types
+      keys = keys.map {|key| key.is_a?(Model::QualifiedReference) ? @@type_parser.interpret(key, scope) : evaluate(key, scope) }
+    else
+      keys = keys.map {|key| evaluate(key, scope) }
+      # Resource[File] becomes File
+      return keys[0] if Types::PResourceType::DEFAULT == left && keys.size == 1 && keys[0].is_a?(Types::PResourceType)
+    end
+    AccessOperator.new(o).access(left, scope, *keys)
   end
 
   # Evaluates <, <=, >, >=, and ==
@@ -411,19 +466,14 @@ class Puppet::Pops::Evaluator::EvaluatorImpl
     left = evaluate(o.left_expr, scope)
     right = evaluate(o.right_expr, scope)
 
-    @migration_checker.report_uc_bareword_type(left, o.left_expr)
-    @migration_checker.report_uc_bareword_type(right, o.right_expr)
-
     begin
     # Left is a type
-    if left.is_a?(Puppet::Pops::Types::PAnyType)
+    if left.is_a?(Types::PAnyType)
       case o.operator
       when :'=='
-        @migration_checker.report_equality_type_mismatch(left, right, o)
         @@type_calculator.equals(left,right)
 
       when :'!='
-        @migration_checker.report_equality_type_mismatch(left, right, o)
         !@@type_calculator.equals(left,right)
 
       when :'<'
@@ -444,10 +494,8 @@ class Puppet::Pops::Evaluator::EvaluatorImpl
     else
       case o.operator
       when :'=='
-        @migration_checker.report_equality_type_mismatch(left, right, o)
         @@compare_operator.equals(left,right)
       when :'!='
-        @migration_checker.report_equality_type_mismatch(left, right, o)
         ! @@compare_operator.equals(left,right)
       when :'<'
         @@compare_operator.compare(left,right) < 0
@@ -492,16 +540,20 @@ class Puppet::Pops::Evaluator::EvaluatorImpl
     left = evaluate(o.left_expr, scope)
     pattern = evaluate(o.right_expr, scope)
 
-    @migration_checker.report_uc_bareword_type(left, o.left_expr)
-
     # matches RHS types as instance of for all types except a parameterized Regexp[R]
-    if pattern.is_a?(Puppet::Pops::Types::PAnyType)
+    if pattern.is_a?(Types::PAnyType)
       # evaluate as instance? of type check
-      matched = @@type_calculator.instance?(pattern, left)
+      matched = pattern.instance?(left)
       # convert match result to Boolean true, or false
       return o.operator == :'=~' ? !!matched : !matched
     end
 
+    if pattern.is_a?(Semantic::VersionRange)
+      # evaluate if range includes version
+      matched = Types::PSemVerRangeType.include?(pattern, left)
+      return o.operator == :'=~' ? matched : !matched
+    end
+
     begin
       pattern = Regexp.new(pattern) unless pattern.is_a?(Regexp)
     rescue StandardError => e
@@ -523,8 +575,6 @@ class Puppet::Pops::Evaluator::EvaluatorImpl
   def eval_InExpression o, scope
     left = evaluate(o.left_expr, scope)
     right = evaluate(o.right_expr, scope)
-    @migration_checker.report_uc_bareword_type(left, o.left_expr)
-    @migration_checker.report_in_expression(o)
     @@compare_operator.include?(right, left, scope)
   end
 
@@ -578,9 +628,8 @@ class Puppet::Pops::Evaluator::EvaluatorImpl
     # memo scope level before evaluating test - don't want a match in the case test to leak $n match vars
     # to expressions after the case expression.
     #
-    with_guarded_scope(scope) do
+    scope.with_guarded_scope do
       test = evaluate(o.test, scope)
-      @migration_checker.report_uc_bareword_type(test, o.test)
 
       result = nil
       the_default = nil
@@ -589,10 +638,10 @@ class Puppet::Pops::Evaluator::EvaluatorImpl
         if co.values.find do |c|
           c = unwind_parentheses(c)
           case c
-          when Puppet::Pops::Model::LiteralDefault
+          when Model::LiteralDefault
             the_default = co.then_expr
             next false
-          when Puppet::Pops::Model::UnfoldExpression
+          when Model::UnfoldExpression
             # not ideal for error reporting, since it is not known which unfolded result
             # that caused an error - the entire unfold expression is blamed (i.e. the var c, passed to is_match?)
             evaluate(c, scope).any? {|v| is_match?(test, v, c, co, scope) }
@@ -616,7 +665,10 @@ class Puppet::Pops::Evaluator::EvaluatorImpl
   # to the compiler to collect the resources specified by the query.
   #
   def eval_CollectExpression o, scope
-    Puppet::Pops::Evaluator::CollectorTransformer.new().transform(o,scope)
+    if o.query.is_a?(Model::ExportedQuery)
+      optionally_fail(Issues::RT_NO_STORECONFIGS, o);
+    end
+    CollectorTransformer.new().transform(o,scope)
   end
 
   def eval_ParenthesizedExpression(o, scope)
@@ -647,47 +699,50 @@ class Puppet::Pops::Evaluator::EvaluatorImpl
 
     # Get the type name
     type_name =
-    if (tmp_name = o.type_name).is_a?(Puppet::Pops::Model::QualifiedName)
+    if (tmp_name = o.type_name).is_a?(Model::QualifiedName)
       tmp_name.value # already validated as a name
     else
       type_name_acceptable =
       case o.type_name
-      when Puppet::Pops::Model::QualifiedReference
+      when Model::QualifiedReference
         true
-      when Puppet::Pops::Model::AccessExpression
-        o.type_name.left_expr.is_a?(Puppet::Pops::Model::QualifiedReference)
+      when Model::AccessExpression
+        o.type_name.left_expr.is_a?(Model::QualifiedReference)
       end
 
       evaluated_name = evaluate(tmp_name, scope)
       unless type_name_acceptable
-        actual = type_calculator.generalize!(type_calculator.infer(evaluated_name)).to_s
-        fail(Puppet::Pops::Issues::ILLEGAL_RESOURCE_TYPE, o.type_name, {:actual => actual})
+        actual = type_calculator.generalize(type_calculator.infer(evaluated_name)).to_s
+        fail(Issues::ILLEGAL_RESOURCE_TYPE, o.type_name, {:actual => actual})
       end
 
       # must be a CatalogEntry subtype
       case evaluated_name
-      when Puppet::Pops::Types::PHostClassType
+      when Types::PHostClassType
         unless evaluated_name.class_name.nil?
-          fail(Puppet::Pops::Issues::ILLEGAL_RESOURCE_TYPE, o.type_name, {:actual=> evaluated_name.to_s})
+          fail(Issues::ILLEGAL_RESOURCE_TYPE, o.type_name, {:actual=> evaluated_name.to_s})
         end
         'class'
 
-      when Puppet::Pops::Types::PResourceType
+      when Types::PResourceType
         unless evaluated_name.title().nil?
-          fail(Puppet::Pops::Issues::ILLEGAL_RESOURCE_TYPE, o.type_name, {:actual=> evaluated_name.to_s})
+          fail(Issues::ILLEGAL_RESOURCE_TYPE, o.type_name, {:actual=> evaluated_name.to_s})
         end
         evaluated_name.type_name # assume validated
 
+      when Types::PTypeReferenceType
+        fail(Issues::UNKNOWN_RESOURCE_TYPE, o.type_string, {:type_name => evaluated_name.to_s})
+
       else
-        actual = type_calculator.generalize!(type_calculator.infer(evaluated_name)).to_s
-        fail(Puppet::Pops::Issues::ILLEGAL_RESOURCE_TYPE, o.type_name, {:actual=>actual})
+        actual = type_calculator.generalize(type_calculator.infer(evaluated_name)).to_s
+        fail(Issues::ILLEGAL_RESOURCE_TYPE, o.type_name, {:actual=>actual})
       end
     end
 
     # This is a runtime check - the model is valid, but will have runtime issues when evaluated
     # and storeconfigs is not set.
     if(o.exported)
-      optionally_fail(Puppet::Pops::Issues::RT_NO_STORECONFIGS_EXPORT, o);
+      optionally_fail(Issues::RT_NO_STORECONFIGS_EXPORT, o);
     end
 
     titles_to_body = {}
@@ -705,24 +760,24 @@ class Puppet::Pops::Evaluator::EvaluatorImpl
       # There may be a :default entry, its entries apply with lower precedence
       #
       if titles.nil?
-        fail(Puppet::Pops::Issues::MISSING_TITLE, body.title)
+        fail(Issues::MISSING_TITLE, body.title)
       end
       titles = [titles].flatten
 
       # Check types of evaluated titles and duplicate entries
       titles.each_with_index do |title, index|
         if title.nil?
-          fail(Puppet::Pops::Issues::MISSING_TITLE_AT, body.title, {:index => index})
+          fail(Issues::MISSING_TITLE_AT, body.title, {:index => index})
 
         elsif !title.is_a?(String) && title != :default
-          actual = type_calculator.generalize!(type_calculator.infer(title)).to_s
-          fail(Puppet::Pops::Issues::ILLEGAL_TITLE_TYPE_AT, body.title, {:index => index, :actual => actual})
+          actual = type_calculator.generalize(type_calculator.infer(title)).to_s
+          fail(Issues::ILLEGAL_TITLE_TYPE_AT, body.title, {:index => index, :actual => actual})
 
         elsif title == EMPTY_STRING
-         fail(Puppet::Pops::Issues::EMPTY_STRING_TITLE_AT, body.title, {:index => index})
+         fail(Issues::EMPTY_STRING_TITLE_AT, body.title, {:index => index})
 
         elsif titles_to_body[title]
-          fail(Puppet::Pops::Issues::DUPLICATE_TITLE, o, {:title => title})
+          fail(Issues::DUPLICATE_TITLE, o, {:title => title})
         end
         titles_to_body[title] = body
       end
@@ -739,7 +794,7 @@ class Puppet::Pops::Evaluator::EvaluatorImpl
         params = [params] unless params.is_a?(Array)
         params.each do |p|
           if param_memo.include? p.name
-            fail(Puppet::Pops::Issues::DUPLICATE_ATTRIBUTE, o, {:attribute => p.name})
+            fail(Issues::DUPLICATE_ATTRIBUTE, o, {:attribute => p.name})
           end
           param_memo[p.name] = p
         end
@@ -774,8 +829,8 @@ class Puppet::Pops::Evaluator::EvaluatorImpl
   def eval_AttributesOperation(o, scope)
     hashed_params = evaluate(o.expr, scope)
     unless hashed_params.is_a?(Hash)
-      actual = type_calculator.generalize!(type_calculator.infer(hashed_params)).to_s
-      fail(Puppet::Pops::Issues::TYPE_MISMATCH, o.expr, {:expected => 'Hash', :actual => actual})
+      actual = type_calculator.generalize(type_calculator.infer(hashed_params)).to_s
+      fail(Issues::TYPE_MISMATCH, o.expr, {:expected => 'Hash', :actual => actual})
     end
     hashed_params.map { |k,v| create_resource_parameter(o, scope, k, v, :'=>') }
   end
@@ -785,10 +840,10 @@ class Puppet::Pops::Evaluator::EvaluatorImpl
   def eval_ResourceDefaultsExpression(o, scope)
     type = evaluate(o.type_ref, scope)
     type_name =
-    if type.is_a?(Puppet::Pops::Types::PResourceType) && !type.type_name.nil? && type.title.nil?
+    if type.is_a?(Types::PResourceType) && !type.type_name.nil? && type.title.nil?
       type.type_name # assume it is a valid name
     else
-      actual = type_calculator.generalize!(type_calculator.infer(type))
+      actual = type_calculator.generalize(type_calculator.infer(type))
       fail(Issues::ILLEGAL_RESOURCE_TYPE, o.type_ref, {:actual => actual})
     end
     evaluated_parameters = o.operations.map {|op| evaluate(op, scope) }
@@ -800,12 +855,23 @@ class Puppet::Pops::Evaluator::EvaluatorImpl
   # Evaluates function call by name.
   #
   def eval_CallNamedFunctionExpression(o, scope)
+    # If LHS is a type (i.e. Integer, or Integer[...]
+    # the call is taken as an instantiation of the given type
+    #
+    functor = o.functor_expr
+    if functor.is_a?(Model::QualifiedReference) ||
+      functor.is_a?(Model::AccessExpression) && functor.left_expr.is_a?(Model::QualifiedReference)
+      # instantiation
+      type = evaluate(functor, scope)
+      return call_function_with_block('new', unfold([type], o.arguments || [], scope), o, scope)
+    end
+
     # The functor expression is not evaluated, it is not possible to select the function to call
     # via an expression like $a()
-    case o.functor_expr
-    when Puppet::Pops::Model::QualifiedName
+    case functor
+    when Model::QualifiedName
       # ok
-    when Puppet::Pops::Model::RenderStringExpression
+    when Model::RenderStringExpression
       # helpful to point out this easy to make Epp error
       fail(Issues::ILLEGAL_EPP_PARAMETERS, o)
     else
@@ -818,15 +884,26 @@ class Puppet::Pops::Evaluator::EvaluatorImpl
   # Evaluation of CallMethodExpression handles a NamedAccessExpression functor (receiver.function_name)
   #
   def eval_CallMethodExpression(o, scope)
-    unless o.functor_expr.is_a? Puppet::Pops::Model::NamedAccessExpression
+    unless o.functor_expr.is_a? Model::NamedAccessExpression
       fail(Issues::ILLEGAL_EXPRESSION, o.functor_expr, {:feature=>'function accessor', :container => o})
     end
     receiver = unfold([], [o.functor_expr.left_expr], scope)
     name = o.functor_expr.right_expr
-    unless name.is_a? Puppet::Pops::Model::QualifiedName
+    unless name.is_a? Model::QualifiedName
       fail(Issues::ILLEGAL_EXPRESSION, o.functor_expr, {:feature=>'function name', :container => o})
-    end 
+    end
     name = name.value # the string function name
+
+    obj = receiver[0]
+    receiver_type = Types::TypeCalculator.infer(obj)
+    if receiver_type.is_a?(Types::PObjectType)
+      member = receiver_type[name]
+      unless member.nil?
+        args = unfold([], o.arguments || [], scope)
+        return o.lambda.nil? ? member.invoke(obj, scope, args) : member.invoke(obj, scope, args, &proc_from_lambda(o.lambda, scope))
+      end
+    end
+
     call_function_with_block(name, unfold(receiver, o.arguments || [], scope), o, scope)
   end
 
@@ -834,75 +911,16 @@ class Puppet::Pops::Evaluator::EvaluatorImpl
     if o.lambda.nil?
       call_function(name, evaluated_arguments, o, scope)
     else
-      closure = Puppet::Pops::Evaluator::Closure.new(self, o.lambda, scope)
-      call_function(name, evaluated_arguments, o, scope, &proc_from_closure(closure))
+      call_function(name, evaluated_arguments, o, scope, &proc_from_lambda(o.lambda, scope))
     end
   end
   private :call_function_with_block
 
-  # Creates a Proc with an arity count that matches the parameters of the given closure. The arity will
-  # be correct up to 10 parameters and then default to varargs (-1)
-  #
-  def proc_from_closure(closure)
-    return Puppet::Pops::Evaluator::PuppetProc.new(closure) { |*args| closure.call(*args) } unless RUBY_VERSION[0,3] == '1.8'
-
-    # This code is required since a Proc isn't propagated by reference in Ruby 1.8.x. It produces a standard
-    # Proc that has correct arity as a replacement for the otherwise used PuppetProc
-    # TODO: Remove when Ruby 1.8.x support is dropped
-    arity = closure.parameters.reduce(0) do |memo, param|
-      count = memo + 1
-      break -count if param.captures_rest || !param.value.nil?
-      count
-    end
-
-    case arity
-    when 0
-      proc { || closure.call }
-    when 1
-      proc { |a| closure.call(a) }
-    when 2
-      proc { |a, b| closure.call(a, b) }
-    when 3
-      proc { |a, b, c| closure.call(a, b, c) }
-    when 4
-      proc { |a, b, c, d| closure.call(a, b, c, d) }
-    when 5
-      proc { |a, b, c, d, e| closure.call(a, b, c, d, e) }
-    when 6
-      proc { |a, b, c, d, e, f| closure.call(a, b, c, d, e, f) }
-    when 7
-      proc { |a, b, c, d, e, f, g| closure.call(a, b, c, d, e, f, g) }
-    when 8
-      proc { |a, b, c, d, e, f, g, h| closure.call(a, b, c, d, e, f, g, h) }
-    when 9
-      proc { |a, b, c, d, e, f, g, h, i| closure.call(a, b, c, d, e, f, g, h, i) }
-    when 10
-      proc { |a, b, c, d, e, f, g, h, i, j| closure.call(a, b, c, d, e, f, g, h, i, j) }
-    when -1
-      proc { |*v| closure.call(*v) }
-    when -2
-      proc { |a, *v| closure.call(a, *v) }
-    when -3
-      proc { |a, b, *v| closure.call(a, b, *v) }
-    when -4
-      proc { |a, b, c, *v| closure.call(a, b, c, *v) }
-    when -5
-      proc { |a, b, c, d, *v| closure.call(a, b, c, d, *v) }
-    when -6
-      proc { |a, b, c, d, e, *v| closure.call(a, b, c, d, e, *v) }
-    when -7
-      proc { |a, b, c, d, e, f, *v| closure.call(a, b, c, d, e, f, *v) }
-    when -8
-      proc { |a, b, c, d, e, f, g, *v| closure.call(a, b, c, d, e, f, g, *v) }
-    when -9
-      proc { |a, b, c, d, e, f, g, h, *v| closure.call(a, b, c, d, e, f,g, h, *v) }
-    when -10
-      proc { |a, b, c, d, e, f, g, h, i, *v| closure.call(a, b, c, d, e, f,g, h, i, *v) }
-    else
-      proc { |*a| closure.call(*a) }
-    end
+  def proc_from_lambda(lambda, scope)
+    closure = Closure.new(self, lambda, scope)
+    PuppetProc.new(closure) { |*args| closure.call(*args) }
   end
-  private :proc_from_closure
+  private :proc_from_lambda
 
   # @example
   #   $x ? { 10 => true, 20 => false, default => 0 }
@@ -911,18 +929,17 @@ class Puppet::Pops::Evaluator::EvaluatorImpl
     # memo scope level before evaluating test - don't want a match in the case test to leak $n match vars
     # to expressions after the selector expression.
     #
-    with_guarded_scope(scope) do
+    scope.with_guarded_scope do
       test = evaluate(o.left_expr, scope)
-      @migration_checker.report_uc_bareword_type(test, o.left_expr)
 
       the_default = nil
       selected = o.selectors.find do |s|
         me = unwind_parentheses(s.matching_expr)
         case me
-        when Puppet::Pops::Model::LiteralDefault
+        when Model::LiteralDefault
           the_default = s.value_expr
           false
-        when Puppet::Pops::Model::UnfoldExpression
+        when Model::UnfoldExpression
           # not ideal for error reporting, since it is not known which unfolded result
           # that caused an error - the entire unfold expression is blamed (i.e. the var c, passed to is_match?)
           evaluate(me, scope).any? {|v| is_match?(test, v, me, s, scope) }
@@ -954,7 +971,7 @@ class Puppet::Pops::Evaluator::EvaluatorImpl
 
   # Evaluates Puppet DSL `if`
   def eval_IfExpression o, scope
-    with_guarded_scope(scope) do
+    scope.with_guarded_scope do
       if is_true?(evaluate(o.test, scope), o.test)
         evaluate(o.then_expr, scope)
       else
@@ -965,7 +982,7 @@ class Puppet::Pops::Evaluator::EvaluatorImpl
 
   # Evaluates Puppet DSL `unless`
   def eval_UnlessExpression o, scope
-    with_guarded_scope(scope) do
+    scope.with_guarded_scope do
       unless is_true?(evaluate(o.test, scope), o.test)
         evaluate(o.then_expr, scope)
       else
@@ -1013,7 +1030,7 @@ class Puppet::Pops::Evaluator::EvaluatorImpl
   # @return [String] the interpolated result
   #
   def eval_TextExpression o, scope
-    if o.expr.is_a?(Puppet::Pops::Model::QualifiedName)
+    if o.expr.is_a?(Model::QualifiedName)
       string(get_variable_value(o.expr.value, o, scope), scope)
     else
       string(evaluate(o.expr, scope), scope)
@@ -1045,7 +1062,7 @@ class Puppet::Pops::Evaluator::EvaluatorImpl
   end
 
   def string_PAnyType(o, scope)
-    @@type_calculator.string(o)
+    o.to_s
   end
 
   # Produces concatenation / merge of x and y.
@@ -1117,7 +1134,7 @@ class Puppet::Pops::Evaluator::EvaluatorImpl
         # Use type calcultor to determine if array is Array[Array[?]], and if so use second form
         # of call
         t = @@type_calculator.infer(y)
-        if t.element_type.is_a? Puppet::Pops::Types::PArrayType
+        if t.element_type.is_a? Types::PArrayType
           Hash[y]
         else
           Hash[*y]
@@ -1168,33 +1185,20 @@ class Puppet::Pops::Evaluator::EvaluatorImpl
   # of value except regular expression where a match is performed.
   #
   def is_match?(left, right, o, option_expr, scope)
-    @migration_checker.report_option_type_mismatch(left, right, option_expr, o)
-    if right.is_a?(Puppet::Pops::Types::PAnyType)
-      @migration_checker.report_uc_bareword_type(right, o)
-    end
     @@compare_operator.match(left, right, scope)
   end
 
-  def with_guarded_scope(scope)
-    scope_memo = get_scope_nesting_level(scope)
-    begin
-      yield
-    ensure
-      set_scope_nesting_level(scope, scope_memo)
-    end
-  end
-
   # Maps the expression in the given array to their product except for UnfoldExpressions which are first unfolded.
   # The result is added to the given result Array.
   # @param result [Array] Where to add the result (may contain information to add to)
-  # @param array [Array[Puppet::Pops::Model::Expression] the expressions to map
+  # @param array [Array[Model::Expression] the expressions to map
   # @param scope [Puppet::Parser::Scope] the scope to evaluate in
   # @return [Array] the given result array with content added from the operation
   #
   def unfold(result, array, scope)
     array.each do |x|
       x = unwind_parentheses(x)
-      if x.is_a?(Puppet::Pops::Model::UnfoldExpression)
+      if x.is_a?(Model::UnfoldExpression)
         result.concat(evaluate(x, scope))
       else
         result << evaluate(x, scope)
@@ -1205,8 +1209,10 @@ class Puppet::Pops::Evaluator::EvaluatorImpl
   private :unfold
 
   def unwind_parentheses(o)
-    return o unless o.is_a?(Puppet::Pops::Model::ParenthesizedExpression)
+    return o unless o.is_a?(Model::ParenthesizedExpression)
     unwind_parentheses(o.expr)
   end
   private :unwind_parentheses
 end
+end
+end
