@@ -132,6 +132,10 @@ Puppet::Type.type(:package).provide :openbsd, :parent => Puppet::Provider::Packa
           end
         end
 
+        if @resource[:source][-1,1] != ::File::SEPARATOR
+          raise Puppet::Error,
+                "Please use full PKG_PATH notation instead of '#{@resource[:source]}'"
+        end
         unless @resource[:source]
           raise Puppet::Error,
           "No valid installpath found in /etc/pkg.conf and no source was set"
@@ -167,7 +171,9 @@ Puppet::Type.type(:package).provide :openbsd, :parent => Puppet::Provider::Packa
     # pkg_add(1) doesn't set the return value upon failure so we have to peek
     # at it's output to see if something went wrong.
     output = Puppet::Util.withenv(e_vars) { pkgadd cmd.flatten.compact }
-    if output.nil? or output.size == 0 or output =~ /Can't find /
+    require 'pp'
+    pp output
+    if output =~ /Can't find /
       self.fail "pkg_add returned: #{output.chomp}"
     end
   end
@@ -236,7 +242,7 @@ Puppet::Type.type(:package).provide :openbsd, :parent => Puppet::Provider::Packa
   end
 
   def uninstall_options
-    join_options(resource[:uninstall_options])
+    [join_options(resource[:uninstall_options])]
   end
 
   def uninstall
