@@ -7,7 +7,7 @@ require 'puppet/parameter/package_options'
 require 'puppet/parameter/boolean'
 
 module Puppet
-  newtype(:package) do
+  Type.newtype(:package) do
     @doc = "Manage packages.  There is a basic dichotomy in package
       support right now:  Some package types (e.g., yum and apt) can
       retrieve their own package files, while others (e.g., rpm and sun)
@@ -20,10 +20,19 @@ module Puppet
       requires in order to function, and you must meet those requirements
       to use a given provider.
 
+      You can declare multiple package resources with the same `name`, as long
+      as they specify different providers and have unique titles.
+
+      Note that you must use the _title_ to make a reference to a package
+      resource; `Package[<NAME>]` is not a synonym for `Package[<TITLE>]` like
+      it is for many other resource types.
+
       **Autorequires:** If Puppet is managing the files specified as a
       package's `adminfile`, `responsefile`, or `source`, the package
       resource will autorequire those files."
 
+    feature :reinstallable, "The provider can reinstall packages.",
+      :methods => [:reinstall]
     feature :installable, "The provider can install packages.",
       :methods => [:install]
     feature :uninstallable, "The provider can uninstall packages.",
@@ -66,6 +75,12 @@ module Puppet
         value. On packaging systems that manage configuration files separately
         from "normal" system files, you can uninstall config files by
         specifying `purged` as the ensure value. This defaults to `installed`.
+
+        Version numbers must match the full version to install, including
+        release if the provider uses a release moniker. Ranges or semver
+        patterns are not accepted except for the `gem` package provider. For
+        example, to install the bash package from the rpm
+        `bash-4.1.2-29.el6.x86_64.rpm`, use the string `'4.1.2-29.el6'`.
       EOT
 
       attr_accessor :latest
@@ -136,7 +151,7 @@ module Puppet
             return true unless [:absent, :purged, :held].include?(is)
           when :latest
             # Short-circuit packages that are not present
-            return false if is == :absent or is == :purged
+            return false if is == :absent || is == :purged
 
             # Don't run 'latest' more than about every 5 minutes
             if @latest and ((Time.now.to_i - @lateststamp) / 60) < 5
@@ -167,13 +182,17 @@ module Puppet
 
 
           when :absent
-            return true if is == :absent or is == :purged
+            return true if is == :absent || is == :purged
           when :purged
             return true if is == :purged
           # this handles version number matches and
           # supports providers that can have multiple versions installed
           when *Array(is)
             return true
+          else
+            # We have version numbers, and no match. If the provider has
+            # additional logic, run it here.
+            return provider.insync?(is) if provider.respond_to?(:insync?)
           end
         }
 
@@ -193,14 +212,25 @@ module Puppet
           super(newvalue)
         end
       end
+
+      def change_to_s(currentvalue, newvalue)
+        # Handle transitioning from any previous state to 'purged'
+        return 'purged' if newvalue == :purged
+
+        # Check for transitions from nil/purged/absent to 'created' (any state that is not absent and not purged)
+        return 'created' if (currentvalue.nil? || currentvalue == :absent || currentvalue == :purged) && (newvalue != :absent && newvalue != :purged)
+
+        # The base should handle the normal property transitions
+        super(currentvalue, newvalue)
+      end
     end
 
     newparam(:name) do
       desc "The package name.  This is the name that the packaging
       system uses internally, which is sometimes (especially on Solaris)
-      a name that is basically useless to humans.  If you want to
-      abstract package installation, then you can use aliases to provide
-      a common name to packages:
+      a name that is basically useless to humans.  If a package goes by
+      several names, you can use a single title and then set the name
+      conditionally:
 
           # In the 'openssl' class
           $ssl = $operatingsystem ? {
@@ -208,11 +238,9 @@ module Puppet
             default => openssl
           }
 
-          # It is not an error to set an alias to the same value as the
-          # object name.
-          package { $ssl:
+          package { 'openssl':
             ensure => installed,
-            alias  => openssl
+            name   => $ssl,
           }
 
           . etc. .
@@ -222,12 +250,10 @@ module Puppet
             default => openssh
           }
 
-          # Use the alias to specify a dependency, rather than
-          # having another selector to figure it out again.
-          package { $ssh:
+          package { 'openssh':
             ensure  => installed,
-            alias   => openssh,
-            require => Package[openssl]
+            name    => $ssh,
+            require => Package['openssl'],
           }
 
       "
@@ -240,6 +266,37 @@ module Puppet
       end
     end
 
+    # We call providify here so that we can set provider as a namevar.
+    # Normally this method is called after newtype finishes constructing this
+    # Type class.
+    providify
+    paramclass(:provider).isnamevar
+
+    # We have more than one namevar, so we need title_patterns. However, we
+    # cheat and set the patterns to map to name only and completely ignore
+    # provider. So far, the logic that determines uniqueness appears to just
+    # "Do The Right Thing™" when the provider is explicitly set by the user.
+    #
+    # The following resources will be seen as uniqe by puppet:
+    #
+    #     # Uniqueness Key: ['mysql', nil]
+    #     package{'mysql': }
+    #
+    #     # Uniqueness Key: ['mysql', 'gem']
+    #     package{'gem-mysql':
+    #       name     => 'mysql,
+    #       provider => gem
+    #     }
+    #
+    # This does not handle the case where providers like 'yum' and 'rpm' should
+    # clash. Also, declarations that implicitly use the default provider will
+    # clash with those that explicitly use the default.
+    def self.title_patterns
+      # This is the default title pattern for all types, except hard-wired to
+      # set only name.
+      [ [ /(.*)/m, [ [:name] ] ] ]
+    end
+
     newproperty(:package_settings, :required_features=>:package_settings) do
       desc "Settings that can change the contents or configuration of a package.
 
@@ -354,8 +411,9 @@ module Puppet
     end
 
     newparam(:configfiles) do
-      desc "Whether configfiles should be kept or replaced.  Most packages
-        types do not support this parameter. Defaults to `keep`."
+      desc "Whether to keep or replace modified config files when installing or
+        upgrading a package. This only affects the `apt` and `dpkg` providers.
+        Defaults to `keep`."
 
       defaultto :keep
 
@@ -406,10 +464,12 @@ module Puppet
         key and value pair are interpreted in a provider specific way.  Each
         option will automatically be quoted when passed to the install command.
 
-        On Windows, this is the **only** place in Puppet where backslash
-        separators should be used.  Note that backslashes in double-quoted
-        strings _must_ be double-escaped and backslashes in single-quoted
-        strings _may_ be double-escaped.
+        With Windows packages, note that file paths in an install option must
+        use backslashes. (Since install options are passed directly to the
+        installation command, forward slashes won't be automatically converted
+        like they are in `file` resources.) Note also that backslashes in
+        double-quoted strings _must_ be escaped and backslashes in single-quoted
+        strings _can_ be escaped.
       EOT
     end
 
@@ -439,8 +499,7 @@ module Puppet
     newparam(:allow_virtual, :boolean => true, :parent => Puppet::Parameter::Boolean, :required_features => :virtual_packages) do
       desc 'Specifies if virtual package names are allowed for install and uninstall.'
 
-      # In a future release, this should be defaulted to true
-      defaultto false
+      defaultto true
     end
 
     autorequire(:file) do
@@ -473,5 +532,41 @@ module Puppet
     def present?(current_values)
       super && current_values[:ensure] != :purged
     end
+
+    # This parameter exists to ensure backwards compatibility is preserved.
+    # See https://github.com/puppetlabs/puppet/pull/2614 for discussion.
+    # If/when a metaparameter for controlling how arbitrary resources respond
+    # to refreshing is created, that will supersede this, and this will be
+    # deprecated.
+    newparam(:reinstall_on_refresh) do
+      desc "Whether this resource should respond to refresh events (via `subscribe`,
+        `notify`, or the `~>` arrow) by reinstalling the package. Only works for
+        providers that support the `reinstallable` feature.
+
+        This is useful for source-based distributions, where you may want to
+        recompile a package if the build options change.
+
+        If you use this, be careful of notifying classes when you want to restart
+        services. If the class also contains a refreshable package, doing so could
+        cause unnecessary re-installs.
+
+        Defaults to `false`."
+      newvalues(:true, :false)
+
+      defaultto :false
+    end
+
+    # When a refresh event is triggered, calls reinstall on providers
+    # that support the reinstall_on_refresh parameter.
+    def refresh
+      if provider.reinstallable? &&
+        @parameters[:reinstall_on_refresh].value == :true &&
+        @parameters[:ensure].value != :purged &&
+        @parameters[:ensure].value != :absent &&
+        @parameters[:ensure].value != :held
+
+        provider.reinstall
+      end
+    end
   end
 end
